//=============================================================================
//
//   File : kvi_kvs_corefunctions_sz.cpp
//   Created on Fri 31 Oct 2003 01:52:04 by Szymon Stefanek
//
//   This file is part of the KVIrc IRC client distribution
//   Copyright (C) 2003 Szymon Stefanek <pragma at kvirc dot net>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#define __KVIRC__

#include "kvi_kvs_corefunctions.h"

#include "kvi_kvs_kernel.h"


#include "kvi_window.h"
#include "kvi_out.h"
#include "kvi_locale.h"
#include "kvi_app.h"
#include "kvi_options.h"

namespace KviKvsCoreFunctions
{
	/*
		@doc: window
		@type:
			function
		@title:
			$window
		@short:
			Retrieve the id of a specified window.
		@syntax:
			$window
			$window(<caption text>)
		@description:
			Returns the [b]window id[/b] of the first window that
			has the specified <caption text>.[br]
			If no window matches the specified <caption text>, and invalid
			window id is returned (0).[br]
			If no <caption text> is specified , this function returns the id
			of the current window.[br]
		@examples:
			[example]
				[cmd]echo[/cmd] This is the window with id $window
			[/example]
		@seealso:
			[fnc]$channel[/fnc],
			[fnc]$query[/fnc],
			[fnc]$console[/fnc],
			[doc:window_naming_conventions]Window naming conventions[/doc]
	*/

	_KVS_COREFUNCTION(window)
	{
#warning "FIXME: the window identifiers could be numbers!"
		KviWindow * wnd = c->window();

		if(pParams->count())
		{
			QString szCaption;
			if(pParams->firstAsString(szCaption))
			{
				wnd = g_pApp->findWindowByCaption(szCaption);
				if(!wnd)
				{
					c->warning(__tr2qs("Window with caption %Q not found"),&szCaption);
					return true;
				}
			}
		}

		pRetBuffer->setString(QString(wnd->id()));
		return true;
	}

};

