#ifndef _KVI_KVS_MODULEINTERFACE_H_
#define _KVI_KVS_MODULEINTERFACE_H_
//=============================================================================
//
//   File : kvi_kvs_moduleinterface.h
//   Created on Tue 16 Dec 2003 00:27:54 by Szymon Stefanek
//
//   This file is part of the KVIrc IRC client distribution
//   Copyright (C) 2003 Szymon Stefanek <pragma at kvirc dot net>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================


#include "kvi_settings.h"

#include <qdict.h>

#include "kvi_kvs_runtimecontext.h"

typedef bool (*moduleSimpleCommandExecRoutine)(KviKvsRunTimeContext * c,KviKvsVariantList * pParams,KviKvsSwitchList * pSwitches);
typedef struct _KviKvsModuleSimpleCommandExecRoutine
{
	moduleSimpleCommandExecRoutine proc;
} KviKvsModuleSimpleCommandExecRoutine;

typedef bool (*moduleFunctionExecRoutine)(KviKvsRunTimeContext * c,KviKvsVariantList * pParams,KviKvsVariant * pRetBuffer);
typedef struct _KviKvsModuleFunctionExecRoutine
{
	moduleFunctionExecRoutine proc;
} KviKvsModuleFunctionExecRoutine;

typedef bool (*moduleCallbackCommandExecRoutine)(KviKvsRunTimeContext * c,KviKvsVariantList * pParams,KviKvsSwitchList * pSwitches,const KviKvsScript * pCallback);
typedef struct _KviKvsModuleCallbackCommandExecRoutine
{
	moduleCallbackCommandExecRoutine proc;
} KviKvsModuleCallbackCommandExecRoutine;


class KVIRC_API KviKvsModuleInterface
{
public:
	KviKvsModuleInterface();
	~KviKvsModuleInterface();
protected:
	QDict<KviKvsModuleSimpleCommandExecRoutine>         * m_pModuleSimpleCommandExecRoutineDict;
	QDict<KviKvsModuleFunctionExecRoutine>              * m_pModuleFunctionExecRoutineDict;
	QDict<KviKvsModuleCallbackCommandExecRoutine>       * m_pModuleCallbackCommandExecRoutineDict;
public:
	void registerKvsSimpleCommand(const QString &szCommand,KviKvsModuleSimpleCommandExecRoutine *r)
		{ m_pModuleSimpleCommandExecRoutineDict->replace(szCommand,r); };
	void registerKvsCallbackCommand(const QString &szCommand,KviKvsModuleCallbackCommandExecRoutine *r)
		{ m_pModuleCallbackCommandExecRoutineDict->replace(szCommand,r); };
	void registerKvsFunction(const QString &szFunction,KviKvsModuleFunctionExecRoutine *r)
		{ m_pModuleFunctionExecRoutineDict->replace(szFunction,r); };

	void unregisterKvsSimpleCommand(const QString &szCommand)
		{ m_pModuleSimpleCommandExecRoutineDict->remove(szCommand); };
	void unregisterKvsCallbackCommand(const QString &szCommand)
		{ m_pModuleCallbackCommandExecRoutineDict->remove(szCommand); };
	void unregisterKvsFunction(const QString &szFunction)
		{ m_pModuleFunctionExecRoutineDict->remove(szFunction); };

	void unregisterAllKvsSimpleCommands()
		{ m_pModuleSimpleCommandExecRoutineDict->clear(); };
	void unregisterAllKvsCallbackCommands()
		{ m_pModuleCallbackCommandExecRoutineDict->clear(); };
	void unregisterAllKvsFunctions()
		{ m_pModuleFunctionExecRoutineDict->clear(); };

	KviKvsModuleSimpleCommandExecRoutine * findKvsSimpleCommand(const QString &szCommand)
		{ return m_pModuleSimpleCommandExecRoutineDict->find(szCommand); };
	KviKvsModuleCallbackCommandExecRoutine * findKvsCallbackCommand(const QString &szCommand)
		{ return m_pModuleCallbackCommandExecRoutineDict->find(szCommand); };
	KviKvsModuleFunctionExecRoutine * findKvsFunction(const QString &szFunction)
		{ return m_pModuleFunctionExecRoutineDict->find(szFunction); };
};

#endif //!_KVI_KVS_MODULEINTERFACE_H_
