#ifndef _KVI_KVS_SCRIPT_H_
#define _KVI_KVS_SCRIPT_H_
//=============================================================================
//
//   File : kvi_kvs_script.h
//   Creation date : Thu 25 Sep 2003 05.12 CEST by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 2003 Szymon Stefanek (pragma at kvirc dot net)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//=============================================================================

#include "kvi_settings.h"
#include "kvi_qstring.h"
#include "kvi_window.h"
#include "kvi_list.h"
#include "kvi_kvs_report.h"
#include "kvi_kvs_variant.h"
#include "kvi_kvs_variantlist.h"

#include <qdeepcopy.h>

class KviKvsTreeNodeInstruction;
class KviKvsReport;

class KVIRC_API KviKvsScriptData
{
	friend class KviKvsScript;
protected:
	unsigned int                m_uRefs;     // Reference count for this structure

	QString                     m_szName;    // script context name
	QString                   * m_pszBuffer; // NEVER TOUCH THIS
	const QChar               * m_pBuffer;   // this points to m_szBuffer: use it to extract string data

	KviKvsTreeNodeInstruction * m_pTree;     // syntax tree
	unsigned int                m_uLock;     // this is increased while the script is being executed
};

class KVIRC_API KviKvsScript
{
	friend class KviKvsRunTimeContext;
	friend class KviKvsParser;
protected:
	// this script becomes the owner of pszBuffer!
	KviKvsScript(const QString &szName,QString * pszBuffer,KviKvsTreeNodeInstruction * pPreparsedTree);
public:
	KviKvsScript(const KviKvsScript &src);

	// this script becomes the owner of pszBuffer!
	KviKvsScript(const QString &szName,QString * pszBuffer);
	~KviKvsScript();
private:
	KviKvsScriptData * m_pData;
public:
	const QString & name() const { return m_pData->m_szName; };
	const QString & code() const { return *(m_pData->m_pszBuffer); };
	bool locked() const { return m_pData->m_uLock > 0; };

	enum RunFlags {
		// print the errors
		PrintErrors = 1,
		// print the warnings
		PrintWarnings = 2,
		// PrintErrors | PrintWarnings
		PrintErrorsAndWarnings = 3,
		// print warnings only if there are also errors (must have also PrintErrors or PrintWarnings)
		WarningsOnErrorOnly = 4,
		// do not delete the eventual parameters passed (only execute() and run())
		PreserveParams = 8,
		// assume that the variables are global unless explicitly declared
		AssumeGlobals = 16
	};

	// returns true after a succesfull parsing
	// pOutput is useful only for printing errors
	// if 0 , no errors are printed
	bool parse(KviWindow * pOutput = 0,int iRunFlags = PrintErrorsAndWarnings);
	bool execute(KviWindow * pWnd,KviKvsVariantList * pParams = 0,KviKvsVariant * pRetVal = 0,int iRunFlags = PrintErrorsAndWarnings,KviKvsHash * pExtScopeVars = 0);
	// this is MORE OR LESS equivalent to return parse(pWnd,iRunFlags) ? run(pWnd,pParams,pRetVal,iRunFlags) : false;
	bool run(KviWindow * pWnd,KviKvsVariantList * pParams = 0,KviKvsVariant * pRetVal = 0,int iRunFlags = PrintErrorsAndWarnings,KviKvsHash * pExtScopeVars = 0);
	// this is the return value of the script
	// it is valid only after execute() has returned true
	// even after that it may be still 0 : nothing returned
	//KviKvsVariant * returnValue(){ return m_pReturnValue; };

	void dump(const char * prefix);
protected:
	void printReportList(KviWindow * pOutput,KviPtrList<KviKvsReport> * pList,bool bRunTime,int iRunFlags);
	// this is intended to be used only by KviKvsRunTimeContext
	// this script becomes the owner of the variant
	//void setReturnValue(KviKvsVariant * pRetVal);
};


#endif //!_KVI_KVS_SCRIPT_H_
